using System;
using System.Collections;

namespace Solitare {
   /// <summary>
   /// Class used to manage the deck; primarily used to track the Z-Order of the deck on the
   /// playing field. The lower the Z-Order, the closer to the top the card is.
   /// </summary>
   public class CardArrayList {
      ArrayList cards = new ArrayList();
      
      /// <summary>
      /// Returns the playing card at the given index.
      /// </summary>
      public PlayingCard this[int index] {
         get { return (PlayingCard)cards[index]; }
         set { cards[index] = value; }
      }

      /// <summary>
      /// Adds the given card to the bottom of the Z-Order.
      /// </summary>
      /// <param name="Card">Card to add.</param>
      /// <returns>Zero-based index where card was placed.</returns>
      public int AddCard(PlayingCard Card) {
         return cards.Add(Card);
      }

      /// <summary>
      /// Inserts the Card at the given index.
      /// </summary>
      /// <param name="Index">Index to place card.</param>
      /// <param name="Card">Card to place.</param>
      public void InsertCard(int Index, PlayingCard Card) {
         cards.Insert(Index, Card);
      }

      /// <summary>
      /// Takes the given card, and moves it to the top.
      /// </summary>
      /// <param name="Card">Card to move.</param>
      public void MoveToTop(PlayingCard Card) {
         cards.Remove(Card);
         cards.Insert(0, Card);
      }

      /// <summary>
      /// Takes the card at the given index, and moves it to the top.
      /// </summary>
      /// <param name="Index">Zero-based index of the card to move.</param>
      public void MoveToTop(int Index) {
         PlayingCard card = this[Index];
         MoveToTop(card);
      }
   }
}
