/* TODO:
 * - Cleanup logic on stacking/queuing cards
 * - Cleanup drawing
 *   - Store graphics where ever possible
 *     - Face of each card in separate array
 *     - Back of cards in static in static variable
 *     - Overlay for "selected" (grayed out) in static variable
 *   - Double Buffering
 * - Add new Graphics (backs, cards themselves)
 * - Implement undo logic
 * - Implement auto-play logic.
 * - Might want to turn off highlighting on a move...
 * - Rules also need to check for a highlighted card, and mark the move as
 *   bad if the card doesn't match, even if the is a legal move. (Flips seem to cause a problem.)
 * - Why is a card moved to the stack returing if you click on the card it was covering?
 * - What's wrong with CheckForVictory()?
 */

// TODO Start using seed better, like the ability to restart a game

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using System.Diagnostics;

using Solitare.Rules;

using System.Threading;


namespace Solitare {
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class CardGame : System.Windows.Forms.Form {
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		IRules gameRules;

      Thread autoplayThread;

      StoreUndoMoves _storeUndoMoves;

		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuExit;
		private System.Windows.Forms.MenuItem menuFile;
		private System.Windows.Forms.MenuItem menuNew;
		private System.Windows.Forms.MenuItem menuItem1;
		private PlayingFieldPanel PanelField;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuKlondike;
		private System.Windows.Forms.MenuItem menuKlondike3;

		String chosenGame;

		bool inGame = false;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuThirteens;
      private System.Windows.Forms.MenuItem menuAbout;
      private System.Windows.Forms.MenuItem menuUndo;
      private System.Windows.Forms.MenuItem menuItem5;
      private System.Windows.Forms.MenuItem scoreMenu;
      private System.Windows.Forms.MenuItem autoplayMenu;

		bool shuffle = true;
      bool showMoves = true;

      PlayingStack[] currentStacks = null;

      // Seed to be used; if not -1, this is used in place of any others.
      int useSeed = -1;

      // Seed of game in progress or just finished
      int currentSeed = -1;

      // Name of the game in progress
      String currentGame;

      // Current Statistics
      Statistics currentStats;

      private static Image _largeHeartImage;
      internal static Image HeartLargeImage {
         get { return _largeHeartImage; }
      }

      private static Image _largeDiamondImage;
      internal static Image LargeDiamondImage {
         get { return _largeDiamondImage; }
      }

      private static Image _largeClubImage;
      internal static Image LargeClubImage {
         get { return _largeClubImage; }
      }

      private static Image _largeSpadeImage;
      internal static Image LargeSpadeImage {
         get { return _largeSpadeImage; }
      }


      private static Image _smallHeartImage;
      internal static Image SmallHeartImage {
         get { return _smallHeartImage; }
      }

      private static Image _smallDiamondImage;
      internal static Image SmallDiamondImage {
         get { return _smallDiamondImage; }
      }

      private static Image _smallClubImage;
      internal static Image SmallClubImage {
         get { return _smallClubImage; }
      }

      private static Image _smallSpadeImage;
      internal static Image SmallSpadeImage {
         get { return _smallSpadeImage; }
      }

      private static Image _jackImage;
      internal static Image JackImage {
         get { return _jackImage; }
      }

      private static Image _queenImage;
      private System.Windows.Forms.MenuItem menuYukon;
      private System.Windows.Forms.MenuItem menuStorehouse;
      private System.Windows.Forms.MenuItem restartMenu;
      private System.Windows.Forms.MenuItem justGoMenu;
   
      internal static Image QueenImage {
         get { return _queenImage; }
      }

      private static Image _kingImage;
      internal static Image KingImage {
         get { return _kingImage; }
      }

      public CardGame() {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         Color c = Color.LightPink;

         // Load up resouces
         System.Reflection.Assembly assm = System.Reflection.Assembly.GetExecutingAssembly();
         System.IO.Stream s = assm.GetManifestResourceStream("Solitare.images.background.jpg");
         //         PanelField.BackgroundImage = Image.FromStream(s);
         
         s = assm.GetManifestResourceStream("Solitare.images.club_large.bmp");
         _largeClubImage = Image.FromStream(s);

         s = assm.GetManifestResourceStream("Solitare.images.diamond_large.bmp");
         _largeDiamondImage = Image.FromStream(s);

         s = assm.GetManifestResourceStream("Solitare.images.spade_large.bmp");
         _largeSpadeImage = Image.FromStream(s);

         s = assm.GetManifestResourceStream("Solitare.images.heart_large.bmp");
         _largeHeartImage = Image.FromStream(s);


         s = assm.GetManifestResourceStream("Solitare.images.club_small.bmp");
         _smallClubImage = Image.FromStream(s);

         s = assm.GetManifestResourceStream("Solitare.images.diamond_small.bmp");
         _smallDiamondImage = Image.FromStream(s);

         s = assm.GetManifestResourceStream("Solitare.images.spade_small.bmp");
         _smallSpadeImage = Image.FromStream(s);

         s = assm.GetManifestResourceStream("Solitare.images.heart_small.bmp");
         _smallHeartImage = Image.FromStream(s);


         s = assm.GetManifestResourceStream("Solitare.images.jack.bmp");
         _jackImage= Image.FromStream(s);

         s = assm.GetManifestResourceStream("Solitare.images.queen.bmp");
         _queenImage = Image.FromStream(s);
         
         s = assm.GetManifestResourceStream("Solitare.images.king.bmp");
         _kingImage = Image.FromStream(s);

         
         menuGame_Clicked(menuThirteens, EventArgs.Empty);

         /*
         ArrayList al = new ArrayList();
         Statistics stats = new Statistics();
         stats.GameName = "Klondike";
         al.Add(stats);
         Statistics.WriteStatisticsFile(al);
         */
      }

		public void NewGame(String ChosenGame) {
         // If I'm not doing autoplay, check for new game, otherwise just kill it...
         if (justGoMenu.Checked == false && (autoplayForm == null || autoplayForm.Visible == false) ) {
            if (inGame && gameRules.CheckForVictory() != true && 
               MessageBox.Show(this, 
               "This will cancel your current game! Do you wish to continue?", "New Game?", 
               MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.No) {
               return;
            }
         }
         currentGame = ChosenGame;

			inGame = true;

         _storeUndoMoves = new StoreUndoMoves();
         
			chosenGame = ChosenGame;
         
			while (PanelField.Controls.Count > 0) {
				Control c = PanelField.Controls[0];
				PanelField.Controls.Remove(c);
			}
         
			switch (chosenGame) {
				case "Klondike":
					gameRules = (IRules)(new Klondike(KlondikeType.Klondike) );
					break;
				case "Klondike3":
					gameRules = (IRules)(new Klondike(KlondikeType.Klondike3Draw) );
					break;
				case "Yukon":
               gameRules = (IRules)(new Klondike(KlondikeType.Yukon));
               break;
            case "Thirteens":
					gameRules = (IRules)(new Thirteens() );
					break;
            case "Storehouse":
               gameRules = (IRules)(new Canfield(CanfieldType.Storehouse));
               break;
				default:
					MessageBox.Show("Invalid Game!");
					return;
			}

         gameRules.CardMoveEvent += new CardMoveEventHandler(OnCardMove);

         // TODO Testing out move tree
         gameRules.ValidMoveEvent += new ValidMoveEventHandler(OnValidMove);
         
			PlayingCard[] cards;
			PlayingStack deck;

			gameRules.BuildDeck(out cards, out deck);
         
			ArrayList oldDeck = new ArrayList();
         
			for (int i = 0; i < cards.Length; i++) {
				PlayingCard p = cards[i];
				// TODO Figure out if I really can draw transparent...
				p.BackColor = System.Drawing.Color.Transparent;
				p.Location = new System.Drawing.Point(10, 400);
				p.TabIndex = 0;
				p.MouseMove += new System.Windows.Forms.MouseEventHandler(this.OnMouseMove);
				p.MouseDown += new System.Windows.Forms.MouseEventHandler(this.OnMouseDown);
				p.MouseUp += new System.Windows.Forms.MouseEventHandler(this.OnMouseUp);
				p.Click += new System.EventHandler(this.OnClick);
				p.DoubleClick += new System.EventHandler(this.OnDoubleClick);

            p.IsFaceShowing = false;

				oldDeck.Add(p);

				PanelField.Controls.Add(p);
			}
         

			Random r = new Random();
         int seed = r.Next(Int32.MaxValue);

         if (true) {
            seed = 229297650; // Seed for a winning hand of klondike
         }

         if (false) {
            seed = 841374905; // Seed to good start of hand
         }

         if (useSeed != -1) {
            seed = useSeed;
            useSeed = -1;
         }

         currentSeed = seed;
         
         Trace.WriteLine("Seed:" + seed);
         r = new Random(seed);

			if (shuffle) {
				for (int i = 0; i < 52; i++) {
					int j = r.Next(oldDeck.Count);

					PlayingCard nextCard = (PlayingCard)oldDeck[j];
					if (nextCard == null) {
						Console.WriteLine("Card is null:" + i + ":" + j);
					}

					deck.AddTopCard((PlayingCard)oldDeck[j]);
					oldDeck.RemoveAt(j);
				} 
			} else {
				for (int i = 51; i >= 0; i--) {
					PlayingCard c = (PlayingCard)oldDeck[i];
					deck.AddTopCard(c);
				}
			}

			gameRules.LayoutBoard(PanelField, out currentStacks);

			for (int i = 0; i < currentStacks.Length; i++) {
				currentStacks[i].MouseMove += new System.Windows.Forms.MouseEventHandler(this.OnMouseMove);
				currentStacks[i].MouseDown += new System.Windows.Forms.MouseEventHandler(this.OnMouseDown);
				currentStacks[i].MouseUp += new System.Windows.Forms.MouseEventHandler(this.OnMouseUp);
				currentStacks[i].Click += new System.EventHandler(this.OnClick);
				currentStacks[i].DoubleClick += new System.EventHandler(this.OnDoubleClick);
			}

			PanelField.Invalidate();

         gameRules.GetValidMoves();

         calculateHash();

         setScore();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
         this.menuExit = new System.Windows.Forms.MenuItem();
         this.mainMenu1 = new System.Windows.Forms.MainMenu();
         this.menuFile = new System.Windows.Forms.MenuItem();
         this.menuNew = new System.Windows.Forms.MenuItem();
         this.menuItem1 = new System.Windows.Forms.MenuItem();
         this.menuItem2 = new System.Windows.Forms.MenuItem();
         this.menuKlondike = new System.Windows.Forms.MenuItem();
         this.menuKlondike3 = new System.Windows.Forms.MenuItem();
         this.menuThirteens = new System.Windows.Forms.MenuItem();
         this.menuYukon = new System.Windows.Forms.MenuItem();
         this.menuStorehouse = new System.Windows.Forms.MenuItem();
         this.menuItem3 = new System.Windows.Forms.MenuItem();
         this.menuUndo = new System.Windows.Forms.MenuItem();
         this.autoplayMenu = new System.Windows.Forms.MenuItem();
         this.menuItem5 = new System.Windows.Forms.MenuItem();
         this.menuAbout = new System.Windows.Forms.MenuItem();
         this.scoreMenu = new System.Windows.Forms.MenuItem();
         this.PanelField = new Solitare.PlayingFieldPanel();
         this.restartMenu = new System.Windows.Forms.MenuItem();
         this.justGoMenu = new System.Windows.Forms.MenuItem();
         this.SuspendLayout();
         // 
         // menuExit
         // 
         this.menuExit.Index = 4;
         this.menuExit.Text = "&Exit";
         this.menuExit.Click += new System.EventHandler(this.menuExit_Click);
         // 
         // mainMenu1
         // 
         this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                  this.menuFile,
                                                                                  this.menuItem2,
                                                                                  this.menuItem3,
                                                                                  this.scoreMenu});
         // 
         // menuFile
         // 
         this.menuFile.Index = 0;
         this.menuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                 this.menuNew,
                                                                                 this.restartMenu,
                                                                                 this.justGoMenu,
                                                                                 this.menuItem1,
                                                                                 this.menuExit});
         this.menuFile.Text = "&File";
         // 
         // menuNew
         // 
         this.menuNew.Index = 0;
         this.menuNew.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
         this.menuNew.Text = "&New Game";
         this.menuNew.Click += new System.EventHandler(this.menuNew_Click);
         // 
         // menuItem1
         // 
         this.menuItem1.Index = 3;
         this.menuItem1.Text = "-";
         // 
         // menuItem2
         // 
         this.menuItem2.Index = 1;
         this.menuItem2.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                  this.menuKlondike,
                                                                                  this.menuKlondike3,
                                                                                  this.menuThirteens,
                                                                                  this.menuYukon,
                                                                                  this.menuStorehouse});
         this.menuItem2.Text = "Choose Game";
         // 
         // menuKlondike
         // 
         this.menuKlondike.Checked = true;
         this.menuKlondike.Index = 0;
         this.menuKlondike.Text = "Klondike";
         this.menuKlondike.Click += new System.EventHandler(this.menuGame_Clicked);
         // 
         // menuKlondike3
         // 
         this.menuKlondike3.Index = 1;
         this.menuKlondike3.Text = "Klondie - 3 Card";
         this.menuKlondike3.Click += new System.EventHandler(this.menuGame_Clicked);
         // 
         // menuThirteens
         // 
         this.menuThirteens.Index = 2;
         this.menuThirteens.Text = "Thirteens";
         this.menuThirteens.Click += new System.EventHandler(this.menuGame_Clicked);
         // 
         // menuYukon
         // 
         this.menuYukon.Index = 3;
         this.menuYukon.Text = "Yukon";
         this.menuYukon.Click += new System.EventHandler(this.menuGame_Clicked);
         // 
         // menuStorehouse
         // 
         this.menuStorehouse.Index = 4;
         this.menuStorehouse.Text = "Storehouse";
         this.menuStorehouse.Click += new System.EventHandler(this.menuGame_Clicked);
         // 
         // menuItem3
         // 
         this.menuItem3.Index = 2;
         this.menuItem3.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
                                                                                  this.menuUndo,
                                                                                  this.autoplayMenu,
                                                                                  this.menuItem5,
                                                                                  this.menuAbout});
         this.menuItem3.Text = "Help";
         // 
         // menuUndo
         // 
         this.menuUndo.Index = 0;
         this.menuUndo.Shortcut = System.Windows.Forms.Shortcut.CtrlU;
         this.menuUndo.Text = "&Undo";
         this.menuUndo.Click += new System.EventHandler(this.menuUndo_Click);
         // 
         // autoplayMenu
         // 
         this.autoplayMenu.Index = 1;
         this.autoplayMenu.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
         this.autoplayMenu.Text = "&Autoplay";
         this.autoplayMenu.Click += new System.EventHandler(this.autoplayMenu_Click);
         // 
         // menuItem5
         // 
         this.menuItem5.Index = 2;
         this.menuItem5.Text = "-";
         // 
         // menuAbout
         // 
         this.menuAbout.Index = 3;
         this.menuAbout.Text = "About";
         this.menuAbout.Click += new System.EventHandler(this.menuAbout_Click);
         // 
         // scoreMenu
         // 
         this.scoreMenu.Index = 3;
         this.scoreMenu.Text = "Score";
         // 
         // PanelField
         // 
         this.PanelField.AutoScroll = true;
         this.PanelField.BackColor = System.Drawing.Color.FloralWhite;
         this.PanelField.Dock = System.Windows.Forms.DockStyle.Fill;
         this.PanelField.Name = "PanelField";
         this.PanelField.Size = new System.Drawing.Size(712, 594);
         this.PanelField.TabIndex = 0;
         // 
         // restartMenu
         // 
         this.restartMenu.Index = 1;
         this.restartMenu.Text = "&Restart";
         this.restartMenu.Click += new System.EventHandler(this.restartMenu_Click);
         // 
         // justGoMenu
         // 
         this.justGoMenu.Index = 2;
         this.justGoMenu.Shortcut = System.Windows.Forms.Shortcut.CtrlJ;
         this.justGoMenu.Text = "&Just Go";
         this.justGoMenu.Click += new System.EventHandler(this.justGoMenu_Click);
         // 
         // CardGame
         // 
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.BackColor = System.Drawing.Color.Turquoise;
         this.ClientSize = new System.Drawing.Size(712, 594);
         this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                      this.PanelField});
         this.Menu = this.mainMenu1;
         this.MinimumSize = new System.Drawing.Size(640, 480);
         this.Name = "CardGame";
         this.Text = "Solitare";
         this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.OnMouseDown);
         this.DoubleClick += new System.EventHandler(this.OnDoubleClick);
         this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.OnMouseUp);
         this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.OnMouseMove);
         this.ResumeLayout(false);

      }
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {
         AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(OnUnhandledException);
			Application.Run(new CardGame());
		}

      private static void OnUnhandledException(Object sender, UnhandledExceptionEventArgs e) {
         MessageBox.Show(e.ExceptionObject.ToString() );
      }


		// TODO Abstract out more logic for what to do on Down.
		// TODO Figure out how to draw a moving stack.
		private void OnMouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
         _currentControl = (Control)sender;
			inClick = true;
			if (sender is PlayingCard) {
				_currentCanMove = false;
				PlayingCard p = (PlayingCard)sender;

				_currentCanMove = gameRules.CanCardMove(p);

				if (_currentCanMove) {
					_x = e.X;
					_y = e.Y;

					_currentX = p.Location.X;
					_currentY = p.Location.Y;

					do {
						p.BringToFront();
					} while ( (p = p.UnderCard) != null);
				}
			}
		}

		// TODO Abstract out how to handle a card movement
		private void OnMouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
         if (!_currentCanMove)
				return;

			if (sender != _currentControl) {
				// Don't attempt to handle it if it's not the card I clicked down on
				return;
			}

			// The mouse didn't really move. Bug?
			if (_x == e.X && _y == e.Y)
				return;

			inClick = false;

			if (sender is PlayingCard && e.Button == MouseButtons.Left) {
				PlayingCard p = (PlayingCard)sender;
            
				int count = 0;

				int x = p.Location.X + e.X - _x;
				int y = p.Location.Y + e.Y - _y;

				do {
					p.SetBounds(x, y + (count * 30),
						p.Width, p.Height);
					count++;
				} while ( (p = p.UnderCard) != null);
			}
		}

      
		// TODO Abstract out more logic from here...
      private void OnMouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
         if (sender != _currentControl) {
            // Don't attempt to handle it if it's not the card I clicked down on
            return;
         }

         if (sender is PlayingCard && e.Button == MouseButtons.Left) {
            PlayingCard p = (PlayingCard)sender;

            if (inClick) {
               _storeUndoMoves.StartTrackingMove(calculateHash(), gameRules);
               Console.WriteLine("OnMouseUp - Click");
               gameRules.HandleClickedCard(p);
               _storeUndoMoves.EndTrackingMove(calculateHash(), gameRules);

               gameRules.GetValidMoves();

               if (gameRules.CheckForVictory() ) {
                  GameWon();
               }

               return;
            }

            if (!_currentCanMove)
               return;

            int x = p.Location.X + e.X - _x;
            int y = p.Location.Y + e.Y - _y;

            PlayingStack droppedOnto = null; 
            long area = -1;
            foreach (PlayingStack stack in currentStacks) {
               PlayingCard top = stack.LookTopCard();

               if (p.Stack == stack) continue;

               if (stack.IntersectsWith(p) ) {
                  long a = stack.IntersectingArea(p);

                  if (a > area) {
                     droppedOnto = stack;
                     area = a;
                  }
               }
            }

            MoveStatus goodDrop = MoveStatus.MoveNotApplicable;
            if (droppedOnto != null) {
               Trace.WriteLine("DROPPED:" + droppedOnto.LookTopCard() );
               _storeUndoMoves.StartTrackingMove(calculateHash(), gameRules);
               goodDrop = gameRules.HandleDroppedCard(p, droppedOnto);
               _storeUndoMoves.EndTrackingMove(calculateHash(), gameRules);
            } 
            
            if (droppedOnto != null && goodDrop == MoveStatus.ValidMoveEnd) {
               x = p.Location.X;
               y = p.Location.Y;
            } else {
               x = _currentX;
               y = _currentY;

               // TODO If it's not a good drop, *DRAG* it back to it's original location
               /*
               CardMoveEventArgs cme = new CardMoveEventArgs();
               cme.NewLocation = p.Stack.Location;
               cme.OriginalLocation = new Point(_currentX, _currentY);
               OnCardMove(p, cme);
               */
               p.MoveCards(p.Stack, p.IsFaceShowing);
            }
            int count = 0;
            do {
               p.SetBounds(x, y + (count * 30),
                  p.Width, p.Height);
               count++;
            } while ( (p = p.UnderCard) != null);
         } else if (sender is PlayingStack) {
            _storeUndoMoves.StartTrackingMove(calculateHash(), gameRules);
            gameRules.HandleClickedStack((PlayingStack)sender);
            _storeUndoMoves.EndTrackingMove(calculateHash(), gameRules);
         }
         this.Update();

         if (gameRules.CheckForVictory() ) {
            GameWon();
         }

         setScore();
      }

      private void setScore() {
         scoreMenu.Text = "SCORE: " + gameRules.Score;
      }

		int _x;
		int _y;

		int _currentX;
		int _currentY;

		bool inClick;

		Control _currentControl;
		bool _currentCanMove;

		private void OnClick(object sender, System.EventArgs e) {
		}

      private void SaveCurrentStatistics(String GameName, bool GameWon) {
         Statistics s =  Statistics.LoadGameStatistics(GameName);
         s.GamesPlayed++;
         if (GameWon)
            s.GamesWon++;
         Statistics.SaveGameStatistics(s);
      }
      
      private void GameWon() {
         SaveCurrentStatistics(currentGame, true);
         MessageBox.Show("You Won!", "Victory");
         inGame = false;
      }

      private void OnDoubleClick(object sender, System.EventArgs e) {
         if (sender is PlayingCard) {
            _storeUndoMoves.StartTrackingMove(calculateHash(), gameRules);
            gameRules.HandleDoubleClickedCard((PlayingCard)sender);
            _storeUndoMoves.EndTrackingMove(calculateHash(), gameRules);
         } else if (sender is PlayingStack) {
            _storeUndoMoves.StartTrackingMove(calculateHash(), gameRules);
            gameRules.HandleDoubleClickedStack((PlayingStack)sender);
            _storeUndoMoves.EndTrackingMove(calculateHash(), gameRules);
         }
         if (gameRules.CheckForVictory() ) {
            GameWon();
         }
         setScore();
      }

		private void menuExit_Click(object sender, System.EventArgs e) {
			this.Close();
		}

		private void menuNew_Click(object sender, System.EventArgs e) {
			NewGame(chosenGame);
		}

		private void menuGame_Clicked(object sender, System.EventArgs e) {
			MenuItem m = (MenuItem)sender;
			foreach (MenuItem mi in m.Parent.MenuItems) {
				mi.Checked = false;
			}
			m.Checked = true;
         if (sender == menuKlondike) {
            NewGame("Klondike");
         } else if (sender == menuKlondike3) {
            NewGame("Klondike3");
         } else if (sender == menuThirteens) {
            NewGame("Thirteens");
         } else if (sender == menuYukon) {
            NewGame("Yukon");
         } else if (sender == menuStorehouse) {
            NewGame("Storehouse");
         }
		}

      private void menuAbout_Click(object sender, System.EventArgs e) {
         // TODO Real about information
         MessageBox.Show("About dialog box goes here.");
      }

      private void OnValidMove(IRules game, ValidMoveEventArgs e) {
      }
      
      private void OnCardMove(PlayingCard card, CardMoveEventArgs e) {
         if (showMoves == true) {
            card.BringToFront();
            int diffX = e.NewLocation.X - e.OriginalLocation.X;
            int diffY = e.NewLocation.Y - e.OriginalLocation.Y;

            double dist = System.Math.Sqrt(diffX * diffX + diffY * diffY) / 50;
            // TODO Cleaner way to do this than a lossy cast?
            int steps = (int)dist;

            for (int i = 0; i < steps; i++) {
               card.SetBounds(
                  e.OriginalLocation.X + ( (diffX / steps) * i), 
                  e.OriginalLocation.Y + ( (diffY / steps) * i),
                  card.Width, card.Height);
               card.Invalidate();
               card.Update();

               foreach (Control c in PanelField.Controls) {
                  if (c is PlayingStack && card.Rect.IntersectsWith(c.Bounds) ) {
                     c.Invalidate();
                     PlayingStack s = (PlayingStack)c;
                     if (s.LookTopCard() != null) {
                        s.LookTopCard().Invalidate();
                     }
                  }
               }
            }
         }
      }

      private void menuUndo_Click(object sender, System.EventArgs e) {
         _storeUndoMoves.UndoLastMove();
         setScore();
      }

      private void autoplayMenu_Click(object sender, System.EventArgs e) {
         showAutoplayForm();
      }

      private void showAutoplayForm() {
         if (autoplayForm == null || autoplayForm.Visible == false) {
            autoplayForm = new Autoplay();
            autoplayForm.Show();
            autoplayForm.AutoplayMove += new EventHandler(nextAutoplayStep);
            autoplayForm.SingleStepChanged += new EventHandler(autoPlaySingleStepChanged);
            autoplayForm.NewGameClick += new EventHandler(autoPlayNewGame);
            autoplayForm.AutoplayCanceled += new EventHandler(autoplayCancelled);
         }
         autoplayThread = new Thread(new System.Threading.ThreadStart(autoPlay));
         autoplayThread.IsBackground = true;
         autoplayThread.Start();
      }

      private void autoPlaySingleStepChanged(Object sender, EventArgs e) {
         showMoves = autoplayForm.SingleStep;
      }
      
      private void nextAutoplayStep(Object sender, EventArgs e) {
         lock (autoplayForm) {
            Monitor.Pulse(autoplayForm);
         }
      }

      private void autoplayCancelled(Object sender, EventArgs e) {
         justGoMenu.Checked = false;
      }

      private void autoPlayNewGame(Object sender, EventArgs e) {
         Object[] args = new Object[2];
         args[0] = this;
         args[1] = EventArgs.Empty;
         this.Invoke(new System.EventHandler(autoPlay), args);
      }

      private void autoPlay(Object sender, EventArgs e) {
         Statistics s =  Statistics.LoadGameStatistics(currentGame);
         useSeed = s.LastSeedUsed + 1;
         
         NewGame(currentGame);
      }
      
      private void autoPlay() {
         bool justGoChecked = justGoMenu.Checked;
         int autoplayCount = 1;
         if (justGoChecked) {
            Statistics.AutoplayStats = true;
         }
         do {
            lock (autoplayForm) {
               Statistics s =  Statistics.LoadGameStatistics(currentGame);
               autoplayForm.GamesPlayed = s.GamesPlayed.ToString();
               autoplayForm.GamesWon = s.GamesWon.ToString();
               if (s.GamesPlayed  > 0) {
                  autoplayForm.Percentage = String.Format("{0:f1}%", ((float)s.GamesWon / (float)s.GamesPlayed) * 100);
               }

               autoplayForm.ShowNew = false;
               moveCount = 0;
               showMoves = false;

               allMoves = new Hashtable();

               DateTime startTime = DateTime.Now;
               Trace.WriteLine("Autoplaying game");
               ArrayList validMoves = gameRules.GetValidMoves();

               ReplayMove rm = new ReplayMove();
               rm.Hash = calculateHash();
               rm.ValidMoves = gameRules.GetValidMoves();
         
               autoplayForm.JustGo = justGoMenu.Checked;
            
               if (!autoplayForm.JustGo) {
                  Monitor.Wait(autoplayForm);
               }
            
               bool won = false;
               
               String status;
               if (autoplayForm.Active == false) {
                  status = "Autoplay Canceled";
                  justGoMenu.Checked = false;
               } else if (testBranches(rm) == true) {
                  status = "Victory conditions found";
                  won = true;
               } else {
                  status = "Can't win this game";
                  won = false;
               }

               TimeSpan diff = DateTime.Now - startTime;
               Trace.WriteLine("Time:" + diff);

               Trace.WriteLine("Total Moves Tested:" + moveCount);
               showMoves = true;

               if (justGoMenu.Checked == false || autoplayForm.Active == false) {
                  MessageBox.Show(status);
                  autoplayForm.ShowNew = true;
               } else {
                  useSeed = currentSeed + 1;
                  autoPlayNewGame(this, EventArgs.Empty);
               }
               if (justGoMenu.Checked == true) {
                  s.GamesPlayed++;
                  s.LastSeedUsed = currentSeed;
                  // TODO Make this better reflect stats
                  if (won)
                     s.GamesWon++;
                  Statistics.SaveGameStatistics(s);
               }
            }
            
         } while (justGoMenu.Checked == true && autoplayCount++ < 25);
         Statistics.AutoplayStats = true;
      }

      private int highScore = -52;
      private int moveCount = 0;

      private Autoplay autoplayForm;

      private Hashtable allMoves;
      
      private bool testBranches(ReplayMove rm) {
         if (allMoves.ContainsKey(rm.Hash) ) 
            return false;;
         
         allMoves.Add(rm.Hash, rm);
         ArrayList moveBranches = rm.ValidMoves;
         moveCount += moveBranches.Count;
         if (gameRules.Score > highScore) {
            highScore = gameRules.Score;
         }
         autoplayForm.HighScore = highScore;

         if (moveCount > gameRules.MaxMoves || !autoplayForm.Active) {
            return false;
         }

         foreach (ValidMoveInfo vmi in moveBranches) {
            if (vmi.MoveType == ValidMoveType.SingleCardClick) {
               gameRules.HandleClickedCard(vmi.FirstCard);
            } else if (vmi.MoveType == ValidMoveType.DoubleCardClick) {
               gameRules.HandleClickedCard(vmi.FirstCard);
               gameRules.HandleClickedCard(vmi.SecondCard);
            } else if (vmi.MoveType == ValidMoveType.StackClick) {
               gameRules.HandleClickedStack(vmi.FirstStack);
            } else if (vmi.MoveType == ValidMoveType.SingleCardToStack) {
               gameRules.HandleClickedCard(vmi.FirstCard);
               gameRules.HandleClickedStack(vmi.FirstStack);
            } else {
               Trace.WriteLine("Unhandled Move TYpe:" + vmi.MoveType);
            }

            autoplayForm.Score = gameRules.Score;

           
            if (gameRules.CheckForVictory() ) {
               return true;
            }

            if (autoplayForm.SingleStep) {
               Monitor.Wait(autoplayForm);
            }

            ReplayMove newMove = new ReplayMove();
            rm.Hash = calculateHash();
            rm.ValidMoves = gameRules.GetValidMoves();

            if (testBranches(rm) == false) {
               _storeUndoMoves.UndoLastMove();
            } else {
               return true;
            }

         }
         return false;
      }

      private int calculateHash() {
         int hash = 0;
         int hashRoot = 0;
         foreach (Control c in PanelField.Controls) {
            if (c is PlayingStack) {
               PlayingStack stack = (PlayingStack)c;

               for (int i = 0; i < stack.Cards.Count; i++) {
                  PlayingCard pc = stack.Cards[i];

                  // TODO Change Suite offset to 1
                  int unique = ((int)pc.Number * ((int)pc.Suite + 1) * 
                     (++hashRoot * 128) * (stack.StackId + 1) * (i + 1));

                  hash += unique;
               }
            }
         }
         return hash;
      }

      private void restartMenu_Click(object sender, System.EventArgs e) {
         useSeed = currentSeed;
         NewGame(currentGame);
      }

      private void justGoMenu_Click(object sender, System.EventArgs e) {
         justGoMenu.Checked = true;
         useSeed = 25;
         NewGame(currentGame);
         autoplayMenu_Click(this, EventArgs.Empty);
      }
 	}

   public class ReplayMove {
      int _hash;
      public int Hash {
         get { return _hash; }
         set { _hash = value; }
      }

      ArrayList _validMoves;
      public ArrayList ValidMoves {
         get { return _validMoves; }
         set { _validMoves = value; }
      }
   }



   public class PlayingFieldPanel : Panel {

      private Panel backgroundPanel;
      
      public PlayingFieldPanel() {
         backgroundPanel = new Panel();
         backgroundPanel.Width = this.Width;
         backgroundPanel.Height = this.Height;
      }

      protected override void OnSizeChanged(EventArgs e) {
         backgroundPanel.Width = this.Width;
         backgroundPanel.Height = this.Height;
      }
/*
      protected override void OnPaintBackground( PaintEventArgs e) {
         // e.Graphics.DrawImage(this.BackgroundImage, e.ClipRectangle);
//         e.Graphics.FillRectangle(System.Drawing.Brushes.ForestGreen, e.ClipRectangle);
      }
      
      public static Graphics offGraphics;
      Image offImage;
      Size offDimension;

      
      protected override void OnPaint( PaintEventArgs e) {
         Console.WriteLine("OnPaint:" + e.ClipRectangle);
         if (offGraphics == null) {
            offDimension = this.Size;
            offImage = new Bitmap(offDimension.Width, offDimension.Height, e.Graphics);
            offGraphics = Graphics.FromImage(offImage);
         }

         lock (offGraphics) {
         
            offGraphics.FillRectangle(System.Drawing.Brushes.ForestGreen, e.ClipRectangle);
         
            PaintEventArgs pea = new PaintEventArgs(offGraphics, e.ClipRectangle);

            base.OnPaint(pea);

            //         current.DrawImage(offImage, 0, 0);
            e.Graphics.DrawImage(offImage, e.ClipRectangle, e.ClipRectangle, GraphicsUnit.Pixel);
            //         current.DrawImage(offImage, e.ClipRectangle, e.ClipRectangle.X, e.ClipRectangle.Y, e.ClipRectangle.Width, e.ClipRectangle.Height, GraphicsUnit.Point);
         }
      }
      */
   }
}