using System;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;

using System.Drawing.Drawing2D;
using System.Collections;

namespace Solitare {

   public enum CardSuites {
      /// <summary>
      /// Hearts
      /// </summary>
		Hearts = 0,
		/// <summary>
		/// Diamonds
		/// </summary>
      Diamonds = 1, 
		/// <summary>
		/// Clubs
		/// </summary>
      Clubs = 2, 
		/// <summary>
		/// Spades
		/// </summary>
      Spades = 3
   }

   public enum CardNumbers : int {
      Ace = 1, 
      Two = 2, 
      Three = 3, 
      Four = 4 , 
      Five = 5, 
      Six = 6, 
      Seven = 7, 
      Eight = 8, 
      Nine = 9, 
      Ten = 10, 
      Jack = 11, 
      Queen = 12, 
      King = 13
   }

   public class PlayingCard : PictureBox {
      private static Image faceDownImage;

      internal static Size cardSize = new System.Drawing.Size(90, 130);

		private Image faceUpImage;
		
		private CardSuites _suite;
      internal CardSuites Suite {
         get { return _suite; }
      }

      private CardNumbers _number;
      internal CardNumbers Number {
         get { return _number; }
         set { _number = value; }
      }

      // All cards start face down
      private bool _isFaceShowing = false;
      internal bool IsFaceShowing {
         get { return _isFaceShowing; }
         set {  
            bool current = _isFaceShowing;
            _isFaceShowing = value; 
            Invalidate(); 
            if (current != value) {
               OnCardFlip(this);
            }
         }
      }

      // No card can move by default
      private bool _canMove = false;
      internal bool CanMove {
         get { return _canMove; }
         set { _canMove = value; }
      }

      // No card can flip by default
      private bool _canFlip = false;
      internal bool CanFlip {
         get { return _canFlip; }
         set { _canFlip = value; }
      }

      internal bool IsBlack {
         get { 
            if (_suite == CardSuites.Clubs || _suite == CardSuites.Spades)
               return true;
            else 
               return false;
         }
      }

      internal bool IsRed {
         get { return !IsBlack; }
      }

      private bool _highlighted;
      internal bool Highlighted {
         get { return _highlighted; }
         set { 
            bool i = false;
            if (_highlighted != value) {
               i = true;
            }
            _highlighted = value;
            if (i)
               Invalidate();
         }
      }
      
         public Rectangle Rect {
         get { return new Rectangle(this.Location.X, this.Location.Y, this.Width, this.Height); }
      }

      private PlayingCard _underCard;
      public PlayingCard UnderCard {
         get { return _underCard; }
         set { _underCard = value; }
      }

      private PlayingCard _overCard;
      public PlayingCard OverCard {
         get { return _overCard; }
         set { _overCard = value; }
      }

      public override String ToString() {
         return base.ToString() + ": " + _number.ToString() + ":" + _suite.ToString();
      }

      internal PlayingCard(CardSuites Suite, CardNumbers Number) {
         _suite = Suite;
         _number = Number;

         Size = cardSize;

         if (faceDownImage == null) {
            drawFaceDownImage();
         }

         drawFaceUpImage();

         this.SetStyle(ControlStyles.UserMouse, true);
         this.SetStyle(ControlStyles.FixedHeight, true);
         this.SetStyle(ControlStyles.FixedWidth, true);
         this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
      }

		private void drawFaceDownImage() {
			Bitmap b = new Bitmap(Size.Width, Size.Height);
			Graphics g = Graphics.FromImage(b);
				
			Color backColor = Color.Blue;

			Pen p = new Pen(Color.LightGray);
				
			g.FillRectangle(new SolidBrush(backColor), ClientRectangle);
            
			for (int i = 15; i < this.Width; i += 10) {
				g.DrawLine(p, new Point(i, 0), new Point(i, this.Height));
			}
			for (int j = 15; j < this.Height; j += 10) {
				g.DrawLine(p, new Point(0, j), new Point(this.Width, j));
			}

			p.Color = Color.Black;
			drawBorder(g);
			faceDownImage = Image.FromHbitmap(b.GetHbitmap());
		}
		
      private void drawSuiteImage(Graphics g, Image img, int row, int col) {
         Bitmap b = new Bitmap(img);
         
         Image inverted = Image.FromHbitmap(b.GetHbitmap());
         inverted.RotateFlip(RotateFlipType.Rotate180FlipNone);

         Image toUse = img;
         if (row >= 4)
            toUse = inverted;
         
         int offsetX = 22;
         int offsetY = 25;

         int iconHeight = img.Height;
         int iconWidth = img.Width;

         int panelHeight = this.Height - (offsetY * 2);
         int panelWidth = this.Width - ((offsetX -2)* 2);

         float rows = (panelHeight) / 7;
         float cols = (panelWidth) / 3;

         float x = offsetX + (cols * col);
         float y = offsetY + (rows * row);
         
         g.DrawImage(toUse, x, y, img.Width, img.Height);
      }
      
      private void drawFaceUpImage() {
			Bitmap b = new Bitmap(Size.Width, Size.Height);
			Graphics g = Graphics.FromImage(b);
			
			Pen p = new Pen(Color.Black);
			
			Color backColor = Color.White;
			Color foreColor;
			if (_suite == CardSuites.Hearts || _suite == CardSuites.Diamonds) {
				foreColor  = Color.Red;
			} else {
				foreColor = Color.Black;
			}
			Font font = new Font("Times New Roman", 14);

         g.FillRectangle(new SolidBrush(backColor), ClientRectangle);
			String number = "";
			switch (_number) {
				case CardNumbers.Two:
				case CardNumbers.Three:
				case CardNumbers.Four:
				case CardNumbers.Five:
				case CardNumbers.Six:
				case CardNumbers.Seven:
				case CardNumbers.Eight:
				case CardNumbers.Nine:
				case CardNumbers.Ten:
					number = ((int)_number).ToString();
					break;
				case CardNumbers.Jack:
				case CardNumbers.Queen:
				case CardNumbers.King:
				case CardNumbers.Ace:
					number = (_number.ToString()[0]).ToString();
					break;
			}

         String suite = _suite.ToString()[0].ToString();

         SizeF numSize = g.MeasureString(number, font);

         Trace.WriteLine("NumSize:" + numSize + ":" + number);

         g.DrawString(number, font, new SolidBrush(foreColor), (float)((25 - numSize.Width) / 2), 1);
         drawBorder(g);
         
         Image largeImage = null;
         Image largeImageInverted = null;
         Image smallImage = null;
         switch (_suite) {
            case CardSuites.Hearts:
               largeImage = CardGame.HeartLargeImage;
               smallImage = CardGame.SmallHeartImage;
               break;
            case CardSuites.Diamonds:
               largeImage = CardGame.LargeDiamondImage;
               smallImage = CardGame.SmallDiamondImage;
               break;
            case CardSuites.Clubs:
               largeImage = CardGame.LargeClubImage;
               smallImage = CardGame.SmallClubImage;
               break;
            case CardSuites.Spades:
               largeImage = CardGame.LargeSpadeImage;
               smallImage = CardGame.SmallSpadeImage;
               break;
         }

         switch (_number) {
            case CardNumbers.Jack:
               largeImage = CardGame.JackImage;
               break;
            case CardNumbers.Queen:
               largeImage = CardGame.QueenImage;
               break;
            case CardNumbers.King:
               largeImage = CardGame.KingImage;
               break;
         }

         if (smallImage != null) {
            g.DrawImage(smallImage, new Rectangle(Convert.ToInt32(((25 - smallImage.Width) / 2)), 23, smallImage.Width, smallImage.Height));

            Image i = Image.FromHbitmap(b.GetHbitmap());
            i.RotateFlip(RotateFlipType.Rotate180FlipNone);

            Rectangle r = new Rectangle(0, 40, this.Width, this.Height - 40);

            g.DrawImage(i, r, r.X, r.Y, r.Width, r.Height, GraphicsUnit.Pixel);
         }

         if (largeImage != null) {
            largeImageInverted = largeImage;
            
            int newX = (this.Width - largeImage.Width) / 2;
            int newY = (this.Height - largeImage.Height) / 2;

            switch (_number) {
               case CardNumbers.Ace:
                  // g.DrawImage(largeImage, new Rectangle(newX, newY, largeImage.Width, largeImage.Height));

                  this.drawSuiteImage(g, largeImage, 3, 1);
                  break;
               case CardNumbers.Two:
                  this.drawSuiteImage(g, largeImage, 0, 1);
                  this.drawSuiteImage(g, largeImage, 6, 1);
                  break;
               case CardNumbers.Three:
                  this.drawSuiteImage(g, largeImage, 0, 1);
                  this.drawSuiteImage(g, largeImage, 3, 1);
                  this.drawSuiteImage(g, largeImage, 6, 1);
                  break;
               case CardNumbers.Four:
                  this.drawSuiteImage(g, largeImage, 1, 0);
                  this.drawSuiteImage(g, largeImage, 1, 2);
                  this.drawSuiteImage(g, largeImage, 5, 0);
                  this.drawSuiteImage(g, largeImage, 5, 2);
                  break;
               case CardNumbers.Five:
                  this.drawSuiteImage(g, largeImage, 1, 0);
                  this.drawSuiteImage(g, largeImage, 1, 2);
                  this.drawSuiteImage(g, largeImage, 5, 0);
                  this.drawSuiteImage(g, largeImage, 5, 2);
                  this.drawSuiteImage(g, largeImage, 3, 1);
                  break;
               case CardNumbers.Six:
                  this.drawSuiteImage(g, largeImage, 1, 0);
                  this.drawSuiteImage(g, largeImage, 1, 2);
                  this.drawSuiteImage(g, largeImage, 3, 0);
                  this.drawSuiteImage(g, largeImage, 3, 2);
                  this.drawSuiteImage(g, largeImage, 5, 0);
                  this.drawSuiteImage(g, largeImage, 5, 2);
                  break;
               case CardNumbers.Seven:
                  this.drawSuiteImage(g, largeImage, 1, 0);
                  this.drawSuiteImage(g, largeImage, 1, 2);
                  this.drawSuiteImage(g, largeImage, 2, 1);
                  this.drawSuiteImage(g, largeImage, 3, 0);
                  this.drawSuiteImage(g, largeImage, 3, 2);
                  this.drawSuiteImage(g, largeImage, 5, 0);
                  this.drawSuiteImage(g, largeImage, 5, 2);
                  break;
               case CardNumbers.Eight:
                  this.drawSuiteImage(g, largeImage, 1, 0);
                  this.drawSuiteImage(g, largeImage, 1, 2);
                  this.drawSuiteImage(g, largeImage, 2, 1);
                  this.drawSuiteImage(g, largeImage, 3, 0);
                  this.drawSuiteImage(g, largeImage, 3, 2);
                  this.drawSuiteImage(g, largeImage, 4, 1);
                  this.drawSuiteImage(g, largeImage, 5, 0);
                  this.drawSuiteImage(g, largeImage, 5, 2);
                  break;
               case CardNumbers.Nine:
                  this.drawSuiteImage(g, largeImage, 0, 0);
                  this.drawSuiteImage(g, largeImage, 0, 2);
                  this.drawSuiteImage(g, largeImage, 3, 1);
                  
                  this.drawSuiteImage(g, largeImage, 2, 0);
                  this.drawSuiteImage(g, largeImage, 2, 2);
                  
                  this.drawSuiteImage(g, largeImage, 4, 0);
                  this.drawSuiteImage(g, largeImage, 4, 2);
                  
                  this.drawSuiteImage(g, largeImage, 6, 0);
                  this.drawSuiteImage(g, largeImage, 6, 2);
                  break;
               case CardNumbers.Ten:
                  this.drawSuiteImage(g, largeImage, 0, 0);
                  this.drawSuiteImage(g, largeImage, 0, 2);

                  
                  this.drawSuiteImage(g, largeImage, 2, 0);
                  this.drawSuiteImage(g, largeImage, 2, 2);
                  
                  this.drawSuiteImage(g, largeImage, 4, 0);
                  this.drawSuiteImage(g, largeImage, 4, 2);

                  this.drawSuiteImage(g, largeImage, 6, 0);
                  this.drawSuiteImage(g, largeImage, 6, 2);
                  

                  this.drawSuiteImage(g, largeImage, 1, 1);
                  this.drawSuiteImage(g, largeImage, 5, 1);
                  break;
               case CardNumbers.Jack:
               case CardNumbers.Queen:
               case CardNumbers.King:
                  g.DrawImage(largeImage, newX, newY, largeImage.Width, largeImage.Height);
                  break;
            }

            if (_number != CardNumbers.Jack && _number != CardNumbers.Queen && _number != CardNumbers.King) {
               newX = 20;
               newY = 15;

               g.DrawLine(p,  new Point(newX, newY), new Point(newX, this.Height - newY));
               g.DrawLine(p,  new Point(this.Width -newX, newY), new Point(this.Width - newX, this.Height - newY));

               g.DrawLine(p,  new Point(newX, newY), new Point(this.Width- newX, newY));
               g.DrawLine(p, new Point(newX, this.Height - newY), new Point(this.Width - newX, this.Height - newY));

            }
         }

			faceUpImage = Image.FromHbitmap(b.GetHbitmap());
		}

      public bool IsTopCard {
         get { 
            PlayingCard p = _stack.LookTopCard();
            return _stack.LookTopCard() == this; 
         }
      }
      
      private PlayingStack _stack;
      internal PlayingStack Stack {
         get { return _stack; }
         set { _stack = value; }
      }

      protected override void OnPaint(PaintEventArgs e) {
         Graphics g = e.Graphics;
         Pen p = new Pen(Color.Black, 1);

//         Trace.WriteLine("RECT:" + g.ClipBounds + ":" + this.Suite + ":" + this.Number);

         RectangleF clipRect = g.ClipBounds;
         
         if (_isFaceShowing) {
				g.DrawImage(faceUpImage, 0, 0, this.Width, this.Height);
         } else {
				g.DrawImage(faceDownImage, 0, 0, Size.Width, Size.Height);
         }
			
			if (_highlighted) {
				HatchBrush hb = new HatchBrush(HatchStyle.Percent25, Color.Black, Color.Transparent);
				g.FillRectangle(hb, 0, 0, Size.Width, Size.Height);
			}
      }

		private void drawBorder(Graphics g) {
			Pen p = new Pen(Color.Black);
			for (int i = 0; i < 4; i++) {
				int xy = 0 + i;
				int height = this.Height - 1 - i;
				int width = this.Width - 1 - i;
            
            
				if (i < 2) {
					p.Color = Color.Black;
				} else {
					p.Color = Color.White;
				} 
				g.DrawLine(p, new Point(xy, xy), new Point(xy, height));
				g.DrawLine(p, new Point(xy, xy), new Point(width, xy));

				g.DrawLine(p, new Point(xy, height), new Point(width, height));
				g.DrawLine(p, new Point(width, xy), new Point(width, height));
			}
		}

      /// <summary>
      /// Moves a single card, or stack of cards to a new PlayingStack. If a stack of cards is selected,
      /// it will move them so as to maintain the properties of both stacks.
      /// </summary>
      /// <param name="NewStack">The destination stack</param>
      /// <param name="CurrentMove">The current move list for undo purposes</param>
      internal void MoveCards(PlayingStack NewStack, bool FlipNextCard) {
         PlayingStack startStack = this.Stack;

         if (this.UnderCard == null) {
            NewStack.AddTopCard(this);
         } else {
            System.Collections.Stack s = new System.Collections.Stack();
            PlayingCard top;
            do {
               top = this.Stack.GetTopCard();

               s.Push(top);
            } while (top != this);

            do {
               top = (PlayingCard)s.Pop();
               NewStack.AddTopCard(top);
            } while (s.Count > 0);
         }

         PlayingCard newTop = startStack.LookTopCard();
         if (FlipNextCard && newTop != null) {
            newTop.IsFaceShowing = true;
         }
      }
      
      /// <summary>
      /// Is this Card and OtherCard the same Suite.
      /// </summary>
      /// <param name="OtherCard">Card to test against.</param>
      /// <returns>true if same suite</returns>
      internal bool IsSameSuite(PlayingCard OtherCard) {
         if (OtherCard == null) return false;
         return (this.Suite == OtherCard.Suite);
      }

      /// <summary>
      /// Is this Card and OtherCard the same Color.
      /// </summary>
      /// <param name="OtherCard">Card to test against.</param>
      /// <returns>true if same color</returns>
      internal bool IsSameColor(PlayingCard OtherCard) {
         if (OtherCard == null) return false;
         return (this.IsBlack == OtherCard.IsBlack);
      }

      /// <summary>
      /// Is this Card one number Higher than OtherCard.
      /// </summary>
      /// <param name="OtherCard">Card to test against.</param>
      /// <returns>true if one higher</returns>
      internal bool IsOneHigher(PlayingCard OtherCard) {
         if (OtherCard == null) return false;
         return ((Convert.ToInt32(this.Number) -
            Convert.ToInt32(OtherCard.Number)) == 1);
      }

      /// <summary>
      /// Is this Card one number Lower than OtherCard.
      /// </summary>
      /// <param name="OtherCard">Card to test against.</param>
      /// <returns>true if one lower</returns>
      internal bool IsOneLower(PlayingCard OtherCard) {
         if (OtherCard == null) return false;
         return ((Convert.ToInt32(this.Number) -
            Convert.ToInt32(OtherCard.Number)) == -1);
      }


      /*
      protected override void OnInvalidated(InvalidateEventArgs e) {
         Trace.WriteLine("Invalidated:" + this.Suite + ":" + this.Number);
      }
      */

      public static event EventHandler CardFlipEvent;

      public static void OnCardFlip(PlayingCard Card) {
         CardFlipEvent(Card, EventArgs.Empty);
      }
   }
}