using System;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;

namespace Solitare.Rules {
   
   public enum CanfieldType {
      Storehouse
   }
   
   /// <summary>
   /// Card games based on Canfield.
   /// </summary>
   public class Canfield : IRules {
      PlayingStack deck;
      PlayingStack discard;

      PlayingStack[] tableu;
      PlayingStack[] foundations;

      PlayingCard[] fullDeck;

      PlayingStack storehouse;

      public Canfield(CanfieldType CanfieldType) {
      }

      public override void BuildDeck(out PlayingCard[] Cards, out PlayingStack Deck) {
         deck = new PlayingStack(0);

         Deck = deck;
         deck.Location = new System.Drawing.Point(10, 10);

         deck.Type = StackTypes.Deck;

         fullDeck = new PlayingCard[52];

         int count = 0;
         for (int i = 0; i < 4; i++) {
            for (int j = 1; j <= 13; j++) {
               PlayingCard p = new PlayingCard((CardSuites)i, (CardNumbers)j);
               p.Name = p.Number.ToString() + "_" + p.Suite;
               fullDeck[count++] = p;
            }
         }

         Cards = fullDeck;

         GameSpecificInfo = new CanfieldSpecific();
         ((CanfieldSpecific)GameSpecificInfo).RedrawCount = 0;
      }

      public override void LayoutBoard(Panel Board, out PlayingStack[] PlayingStacks) {
         PlayingStacks = new PlayingStack[11];
         PlayingStacks[0] = deck;

         Board.Controls.Add(deck);

         discard= new PlayingStack(1);
         discard.DrawTiled = false;
         discard.Location = new System.Drawing.Point(110, 10);
         discard.Type = StackTypes.Discard;
         Board.Controls.Add(discard);

         PlayingStacks[1] = discard;

         foundations = new PlayingStack[4];
         for (int i = 0; i < 4; i++) {
            foundations[i] = new PlayingStack(2+i);
            foundations[i].DrawTiled = false;
            foundations[i].Location = new System.Drawing.Point(310 + 100 * i, 10);
            foundations[i].Type = StackTypes.Foundation;
            Board.Controls.Add(foundations[i]);

            PlayingStacks[i+2] = foundations[i];
         }

         ArrayList twos = new ArrayList();

         for (int i = 0; i < fullDeck.Length; i++) {
            PlayingCard card = fullDeck[i];
            if (card.Number == CardNumbers.Two) {
               twos.Add(card);
            }
         }

         for (int i = 0; i < 4; i++) {
            PlayingCard card = (PlayingCard)twos[i];
            bool temp = deck.RemoveCard(card);
            Trace.WriteLine("TEMP:" + temp);
            card.IsFaceShowing = true;

            Trace.WriteLine("Adding Card:" + card);

            switch (card.Suite) {
               case CardSuites.Diamonds:
                  foundations[1].AddTopCard(card);
                  break;
               case CardSuites.Hearts:
                  foundations[0].AddTopCard(card);
                  break;
               case CardSuites.Spades:
                  foundations[2].AddTopCard(card);
                  break;
               case CardSuites.Clubs:
                  foundations[3].AddTopCard(card);
                  break;
               default:
                     Trace.WriteLine("Falling to default:" + card);
                  break;
            }
         }

         for (int i = 0; i < fullDeck.Length; i++) {
            PlayingCard card = fullDeck[i];
            if (card.Number == CardNumbers.Two) {
               Trace.WriteLine("Shouldn't be here:" + card);
            }
         }

         storehouse = new PlayingStack(6);
         storehouse.DrawTiled = true;
         storehouse.Location = new System.Drawing.Point(110, 150);
         storehouse.Type = StackTypes.Other;
         Board.Controls.Add(storehouse);
         PlayingStacks[6] = storehouse;

         tableu = new PlayingStack[4];
         for (int i = 0; i < 4; i++) {
            tableu[i] = new PlayingStack(7+i);
            tableu[i].DrawTiled = true;
            tableu[i].Location = new System.Drawing.Point(310 + 100 * i, 150);
            tableu[i].Type = StackTypes.Tableu;
            Board.Controls.Add(tableu[i]);

            PlayingStacks[7+i] = tableu[i];
         }

         for (int i = 0; i < 13; i++) {
            PlayingCard card = deck.GetTopCard();
            card.IsFaceShowing = false;

            storehouse.AddTopCard(card);
         }

         storehouse.LookTopCard().IsFaceShowing = true;

         for (int i = 0; i < 4; i++) {
            PlayingCard card = deck.GetTopCard();
            card.IsFaceShowing = true;

            tableu[i].AddTopCard(card);
         }

      }
   
      public override bool CanCardMove(PlayingCard Card) {
         PlayingStack stack = Card.Stack;
         switch (stack.Type) {
            case StackTypes.Discard:
            case StackTypes.Foundation:
            case StackTypes.Tableu:
               return true;
            case StackTypes.Other:
               if (Card == stack.LookTopCard() ) {
                  return true;
               } else {
                  return false;
               }
            default:
               break;
         }
         return false;
      }

      public override MoveStatus HandleClickedCard(PlayingCard Card) {
         return HandleClickedStack(Card.Stack);
      }

      private ArrayList moveList = new ArrayList();

      public override MoveStatus HandleClickedStack(PlayingStack Stack) {
         PlayingCard topCard = Stack.LookTopCard();

         if (Stack == deck) {
            if (Stack.Cards.Count == 0) {
               if ( ((CanfieldSpecific)GameSpecificInfo).RedrawCount == 2) {
                  MessageBox.Show("No more redraws");
                  return MoveStatus.Invalid;
               }
               while (discard.Cards.Count > 0) {
                  PlayingCard top = discard.LookTopCard();
                  top.IsFaceShowing = false;
                  top.MoveCards(deck, false);
               }
               ((CanfieldSpecific)GameSpecificInfo).RedrawCount++;
               return MoveStatus.ValidMoveEnd;
            } else {
               if (topCard != null) {
                  topCard.IsFaceShowing = true;

                  topCard.MoveCards(discard, false);
                  return MoveStatus.ValidMoveEnd;
               }
            }
            return MoveStatus.Invalid;
         }

         return MoveStatus.Invalid;
      }

      public override MoveStatus HandleDoubleClickedCard(PlayingCard Card) {
         for (int i = 0; i < 4; i++) {
            PlayingCard topCard = foundations[i].LookTopCard();

            if (topCard != null && topCard.IsSameSuite(Card) &&
               (topCard.IsOneLower(Card) || 
               (topCard.Number == CardNumbers.King && Card.Number == CardNumbers.Ace))) {
            
               PlayingStack startingStack = Card.Stack;
               topCard.Stack.AddTopCard(Card);

               if (startingStack.Type == StackTypes.Tableu && 
                  startingStack.CardCount == 0 &&
                  storehouse.CardCount > 0) {
                  // Move a card from the storehouse
                  topCard = storehouse.LookTopCard();

                  topCard.MoveCards(startingStack, true);
               }

               if (startingStack == storehouse &&
                  storehouse.CardCount > 0) {
                  topCard = storehouse.LookTopCard();
                  topCard.IsFaceShowing = true;
               }
               return MoveStatus.ValidMoveEnd;
            }

         }
         return MoveStatus.MoveNotApplicable;
      }

      public override MoveStatus HandleDoubleClickedStack(PlayingStack Stack) {
         return MoveStatus.MoveNotApplicable;
      }

      public override MoveStatus HandleDroppedCard(PlayingCard Card,PlayingStack OntoStack) {
         PlayingStack startingStack = Card.Stack;
         
         PlayingCard topCard = OntoStack.LookTopCard();
         
         if (OntoStack.Type == StackTypes.Foundation && Card.IsSameSuite(topCard) &&
            (Card.IsOneHigher(topCard) || topCard.Number == CardNumbers.King &&
            Card.Number == CardNumbers.Ace) ) {
            Trace.WriteLine("Good foundation");

            Card.MoveCards(OntoStack, true);

            if (startingStack.Type == StackTypes.Tableu &&
               startingStack.Cards.Count == 0 && storehouse.Cards.Count > 0) {
               // Move a card from the storehouse
               topCard = storehouse.LookTopCard();
               
               topCard.MoveCards(startingStack, true);
            }

            return MoveStatus.ValidMoveEnd;
         }

         if (OntoStack.Type == StackTypes.Tableu && 
            (topCard == null ||
            (Card.IsSameSuite(topCard) && Card.IsOneLower(topCard) ||
            (topCard.Number == CardNumbers.Ace && Card.Number == CardNumbers.King)))) {

            Card.MoveCards(OntoStack, true);

            if (startingStack.Type == StackTypes.Tableu &&
               startingStack.Cards.Count == 0 && storehouse.Cards.Count > 0) {
               // Move a card from the storehouse
               topCard = storehouse.LookTopCard();
               
               topCard.MoveCards(startingStack, true);
            }

            return MoveStatus.ValidMoveEnd;
         }

         return MoveStatus.Invalid;
      }

      public override bool CheckForVictory() {
         int count = 0;
         for (int i = 0; i < 4; i++) {
            count += foundations[i].CardCount;
         }
         if (count == 52) {
            return true;
         } else {
            return false;
         }
      }

      public override ArrayList GetValidMoves() {
         return new ArrayList();
      }

      public override int Score {
         get { 
            return -50 + (foundations[0].CardCount * 5) +
               foundations[1].CardCount * 5 +
               foundations[2].CardCount * 5 +
               foundations[3].CardCount * 5;
         }
      }

      [Serializable]
      private class CanfieldSpecific {
         int _redrawCount;
         internal int RedrawCount {
            get { return _redrawCount; }
            set { _redrawCount = value; }
         }
      }
   }
}
