using System;
using System.Collections;
using System.Windows.Forms;

using System.Drawing;

using System.Diagnostics;


namespace Solitare.Rules {

   public enum KlondikeType {
      Klondike, Klondike3Draw, Yukon
   }
   
   /// <summary>
   /// Card games based on Klondike
   /// </summary>
	public class Klondike : IRules {
		PlayingStack deck;
		PlayingStack discard;

		PlayingStack[] tableu;
		PlayingStack[] foundations;

		PlayingCard[] fullDeck;
      
		int drawCount = 1;

		PlayingCard selectedCard = null;

      KlondikeType type;
      
		public Klondike(KlondikeType type) {
         if (type == KlondikeType.Klondike3Draw) {
            drawCount = 3;
         }

         this.type = type;
         _maxMoves = 10000;
		}
   
		public override void BuildDeck(out PlayingCard[] Cards, out PlayingStack Deck) {
			deck = new PlayingStack(0);

			Deck = deck;
         deck.Location = new System.Drawing.Point(10, 10);
         deck.Type = StackTypes.Deck;

			fullDeck = new PlayingCard[52];

			int count = 0;
			for (int i = 0; i < 4; i++) {
				for (int j = 1; j <= 13; j++) {
					PlayingCard p = new PlayingCard((CardSuites)i, (CardNumbers)j);
					p.Name = p.Number.ToString() + "_" + p.Suite;
					fullDeck[count++] = p;
				}
			}

			Cards = fullDeck;
		}

      public override void LayoutBoard(Panel Board, out PlayingStack[] PlayingStacks) {
         PlayingStacks = new PlayingStack[12];
         PlayingStacks[0] = deck;

         Board.Controls.Add(deck);

         discard= new PlayingStack(1);
         discard.DrawTiled = false;
         discard.Location = new System.Drawing.Point(110, 10);
         discard.Type = StackTypes.Discard;
         Board.Controls.Add(discard);

         foundations = new PlayingStack[4];
         for (int i = 0; i < 4; i++) {
            foundations[i] = new PlayingStack(2+i);
            foundations[i].DrawTiled = false;
            foundations[i].Location = new System.Drawing.Point(310 + 100 * i, 10);
            foundations[i].Type = StackTypes.Foundation;
            Board.Controls.Add(foundations[i]);

            PlayingStacks[i+1] = foundations[i];
         }

         tableu = new PlayingStack[7];
         for (int i = 0; i < 7; i++) {
            tableu[i] = new PlayingStack(6+i);
            tableu[i].DrawTiled = true;
            tableu[i].Location = new System.Drawing.Point(10 + 100 *i, 200);
            tableu[i].Type = StackTypes.Tableu;
            Board.Controls.Add(tableu[i]);

            PlayingStacks[i+5] = tableu[i];
         }

         for (int i = 0; i < 7; i++) {
            bool first = true;
            for (int j = i; j < 7; j++) {
               PlayingCard c = deck.GetTopCard();
               if (c == null) {
                  Console.WriteLine("HELP!{0}:{1}", i, j);
               } else {
                  if (first) {
                     c.IsFaceShowing = true;
                     first = false;
                  }
                  tableu[j].AddTopCard(c);
               }
            }
         }

         if (type == KlondikeType.Yukon) {

            for (int i = 0; i < 4; i++) {
               for (int j = 1; j < 7; j++) {
                  PlayingCard c = deck.GetTopCard();
                  if (c == null) {
                     Trace.WriteLine(String.Format("Help!!!{0]:{1}", i, j));
                  } else {
                     c.IsFaceShowing = true;
                     tableu[j].AddTopCard(c);
                  }
               }
            }
         }
         

        
      }

		public override bool CanCardMove(PlayingCard Card) {
			if (!Card.IsFaceShowing)
				return false;
         
			if (Card.Stack == deck)
				return false;
         
			if (Card == Card.Stack.LookTopCard() ) {
				return true;
			}

			if (Card.IsFaceShowing)
				return true;
         
			return false;

		}
      
		public override MoveStatus HandleClickedStack(PlayingStack Stack) {
			MoveStatus mv = MoveStatus.Invalid;
			if (selectedCard != null) {
            if ( (Stack.Type == StackTypes.Foundation && Stack.CardCount == 0 && selectedCard.Number == CardNumbers.Ace) ||
               (Stack.Type == StackTypes.Tableu && selectedCard.Number == CardNumbers.King) ) {
               
               selectedCard.MoveCards(Stack, true);
               
               mv = MoveStatus.ValidMoveEnd;
            }
         } else if (Stack == deck) {
				PlayingCard p;
				while ( (p = discard.LookTopCard()) != null) {
               // TODO I'm not quite sure what's the deal here
               
               p.IsFaceShowing = false;

               p.MoveCards(deck, false);

               mv = MoveStatus.ValidMoveEnd;
				}
			}
			
			// TODO Attempt to remove this
			if (selectedCard != null) {
				selectedCard.Highlighted = false;
				selectedCard = null;
			}

			return mv;
		}

		public override MoveStatus HandleDoubleClickedStack(PlayingStack Stack) {
			return MoveStatus.Invalid;
		}

		public override MoveStatus HandleClickedCard(PlayingCard Card) {
         // TODO Make this unecessary
			for (int i = 0; i < fullDeck.Length; i++) {
				fullDeck[i].Highlighted = false;
			}

			MoveStatus mv = MoveStatus.Invalid;

			if (Card.Stack == deck) {
				// If it's the deck, we handle it one way
				for (int i = 0; i < drawCount; i++) {
					PlayingCard newCard = deck.GetTopCard();
					if (newCard != null) {
                  
                  newCard.IsFaceShowing = true;
                  discard.AddTopCard(newCard);
						mv = MoveStatus.ValidMoveEnd;
					}
				}
// TODO Removed for now since it's been invalidated
/*        
			} else if ( Card.IsFaceShowing == false && (Card.CanFlip || Card.IsTopCard) ) {
				Card.IsFaceShowing = true;
				Card.Invalidate(); 
				mv = MoveStatus.ValidMoveEnd;
*/
			} else if (selectedCard == null) {
				selectedCard = Card;
				Card.Highlighted = true;
				mv = MoveStatus.ValidClick;
			} else if (selectedCard == Card) {
				selectedCard.Highlighted = false;
				selectedCard = null;
				mv = MoveStatus.Invalid;
			} else {
				Console.WriteLine("Checking for valid rules");
				PlayingStack stack = Card.Stack;
				if (stack.Type == StackTypes.Foundation) {
					if (Card.Suite == selectedCard.Suite &&
						Card.Number == selectedCard.Number - 1) {
                  
                  selectedCard.MoveCards(Card.Stack, true);

						mv = MoveStatus.ValidMoveEnd;
					}
				} else if (stack.Type == StackTypes.Tableu) {
						if (selectedCard.IsBlack != Card.IsBlack &&
							selectedCard.Number == Card.Number -1) {
                     
                     selectedCard.MoveCards(stack, true);
                  }
					}
				selectedCard.Highlighted = false;
				selectedCard = null;
			}
         if (mv == MoveStatus.ValidMoveEnd) {
            OnValidMove();
         }
         return mv;
		}
	
      
      public override MoveStatus HandleDoubleClickedCard(PlayingCard Card) {
			MoveStatus mv = MoveStatus.Invalid;

         // TODO Make this unecessary
			if (Card.Highlighted) {
            Card.Highlighted = false;
            Card.Invalidate();
         }
         for (int i = 0; i < 4; i++) {
            PlayingCard top = foundations[i].LookTopCard();
            if ( (top == null && Card.Number == CardNumbers.Ace) || 
                 (top != null && top.Suite == Card.Suite && top.Number == Card.Number - 1) ) {
               
               Card.MoveCards(foundations[i], true);
					
               mv = MoveStatus.ValidMoveEnd;
               break;
            }
         }
			return mv;
      }

      public override MoveStatus HandleDroppedCard(PlayingCard Card, PlayingStack OntoStack) {
         PlayingStack s = null;

         PlayingCard topCard = OntoStack.LookTopCard();
         if (OntoStack.Type == StackTypes.Foundation && 
            ( (topCard == null && Card.Number == CardNumbers.Ace) ||
            (topCard != null && topCard.Suite == Card.Suite &&
            Convert.ToInt32(Card.Number) == Convert.ToInt32(topCard.Number) + 1) ) ) {
            s = OntoStack;
         }
         

         if (OntoStack.Type == StackTypes.Tableu &&
            ( (topCard == null && Card.Number  == CardNumbers.King) ||
            (topCard != null && Convert.ToInt32(Card.Number) == Convert.ToInt32(topCard.Number) - 1 &&
            Card.IsBlack != topCard.IsBlack) ) ) {
            s = OntoStack;
         }

         if (s != null) {
            Card.MoveCards(s, true);

            return MoveStatus.ValidMoveEnd;
         } else {
            return MoveStatus.Invalid;
         }
      }

      public override bool CheckForVictory() {
         int count = 0;
         for (int i = 0; i < 4; i++) {
            count += foundations[i].CardCount;
         }
         if (count == 52) {
            return true;
         } else {
            return false;
         }
      }

      public override int Score {
         get { 
            return -50 + (foundations[0].CardCount * 5) +
               foundations[1].CardCount * 5 +
               foundations[2].CardCount * 5 +
               foundations[3].CardCount * 5;
         }
      }
      
      /*
      public override void UndoLastMove() {
#if REMOVE         
         if (_undoStack.Count > 0) {
            ArrayList al = (ArrayList)_undoStack[_undoStack.Count - 1];

            for (int i = al.Count - 1; i >=0; i--) {
               UndoMoveCard umc = (UndoMoveCard)al[i];
      
               switch (umc.UndoType) {
                  case UndoType.FlipCard:
                     umc.StartCard.IsFaceShowing = umc.IsFaceVisibleAtStart;
                     break;

                  case UndoType.MoveSingleCard:
                     // TODO Move Cards *may* not be the best option here
                     umc.StartCard.MoveCards(umc.StartStack, umc.IsFaceVisibleAtStart);
                     umc.StartCard.IsFaceShowing = umc.IsFaceVisibleAtStart;
                     break;
               }
            }
            _undoStack.Remove(al);
         } else {
            // TODO Figure out if this should ever happen
            Trace.WriteLine("No move to undo");
            // MessageBox.Show("No move to undo");
         }
#endif
      }
      */

      private ArrayList getValidMovesForCard(PlayingCard cardToCheck) {
         ArrayList al = new ArrayList();

         if (cardToCheck == null)
            return al;

         if (cardToCheck.Number == CardNumbers.Ace) {
            ValidMoveInfo vmi = new ValidMoveInfo(ValidMoveType.SingleCardToStack);
            vmi.FirstCard = cardToCheck;
            vmi.FirstStack = foundations[(int)cardToCheck.Suite];
            al.Add(vmi);

            return al;
         } else if (cardToCheck.Number == CardNumbers.King) {
            // If it's already at the top, disregard
            if (cardToCheck.OverCard != null) {
               for (int i = 0; i < 7; i++) {
                  if (tableu[i].CardCount == 0) {
                     ValidMoveInfo vmi = new ValidMoveInfo(ValidMoveType.SingleCardToStack);
                     vmi.FirstCard = cardToCheck;
                     vmi.FirstStack = tableu[i];
                     al.Add(vmi);
                     break;
                  }
               }
            }
         }

         PlayingCard topOfStack;
         
         for (int i = 0; i < 7; i++) {
            topOfStack = tableu[i].LookTopCard();
            if (topOfStack != null) {
               if (cardToCheck.IsBlack != topOfStack.IsBlack &&
                  cardToCheck.Number == topOfStack.Number -1) {
                  ValidMoveInfo vmi = new ValidMoveInfo(ValidMoveType.DoubleCardClick);
                  vmi.FirstCard = cardToCheck;
                  vmi.SecondCard = topOfStack;
                  
                  al.Add(vmi);
               }
            }
         }

         topOfStack = foundations[(int)cardToCheck.Suite].LookTopCard();

         if (topOfStack != null) {
            if (topOfStack.Number == cardToCheck.Number -1) {
               ValidMoveInfo vmi = new ValidMoveInfo(ValidMoveType.DoubleCardClick);
               vmi.FirstCard = cardToCheck;
               vmi.SecondCard = topOfStack;

               al.Add(vmi);
            }
         }

         return al;

      }
      
      public override ArrayList GetValidMoves() {
         ArrayList validMoves = new ArrayList();

         PlayingCard current = discard.LookTopCard();
         ArrayList al = getValidMovesForCard(current);
         validMoves.AddRange(al);

         for (int i = 0; i < 7; i++) {
            PlayingCard topCard = tableu[i].LookTopCard();
            al = getValidMovesForCard(topCard);
            validMoves.AddRange(al);
         }


         if (deck.CardCount > 0) {
            ValidMoveInfo vmi = new ValidMoveInfo(ValidMoveType.SingleCardClick);
            vmi.FirstCard = deck.LookTopCard();

            validMoves.Add(vmi);
         } else if ( deck.CardCount == 0 && discard.CardCount > 0) {
            // TODO I'd like to think about making stack click capable of flipping a card on top
            ValidMoveInfo vmi = new ValidMoveInfo(ValidMoveType.StackClick);
            vmi.FirstStack = deck;

            validMoves.Add(vmi);
         }

         return validMoves;
      }
   }
}