using System;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;

using System.Diagnostics;

namespace Solitare.Rules {
	/// <summary>
	/// Pairing game.
	/// </summary>
	public class Thirteens : IRules {
	
		PlayingStack[] tableu;
		PlayingStack deck;
		PlayingStack discard;

		PlayingCard[] fullDeck;
		
		public Thirteens() {		
         _maxMoves = 500;
      }

		public override void BuildDeck(out PlayingCard[] Cards, out PlayingStack Deck) {
			deck = new PlayingStack(0);

			Deck = deck;
         deck.Type = StackTypes.Deck;
			deck.Location = new Point(10, 400);

			fullDeck = new PlayingCard[52];

			int count = 0;
			for (int i = 0; i < 4; i++) {
				for (int j = 1; j <= 13; j++) {
					PlayingCard p = new PlayingCard((CardSuites)i, (CardNumbers)j);
					p.Name = p.Number.ToString() + "_" + p.Suite;
					fullDeck[count++] = p;
				}
			}

			Cards = fullDeck;
		}

		public override void LayoutBoard(Panel Board, out PlayingStack[] PlayingStacks) {
			PlayingStacks = new PlayingStack[10];

			tableu = new PlayingStack[10];

			discard = new PlayingStack(1);

			for (int j = 0; j < 2; j++) {
				for (int i = 0; i < 5; i++) {
					int stack = (j * 5) + i;

					PlayingStack current = new PlayingStack(2+i);
               current.Type = StackTypes.Tableu;
					tableu[stack] = current;
					current.Location = new Point(10 + (100 * i), 20 + (160 * j) );
					
					Board.Controls.Add(current);

					PlayingStacks[stack] = tableu[stack];
				}
			}
			
			discard.Location = new Point(110, 400);
         discard.Type = StackTypes.Discard;
			Board.Controls.Add(discard);

			Board.Controls.Add(deck);

			fillEmptyTableu();
		}

		private void fillEmptyTableu() {
			for (int i = 0; i < 10; i++) {
				if (tableu[i].CardCount == 0 && deck.CardCount > 0) {
               PlayingCard c = deck.LookTopCard();

					c.IsFaceShowing = true;
               OnCardMove(c, tableu[i].Location);
					tableu[i].AddTopCard( c );
				}
			}
		}

		/// <summary>
		/// Determines how best to handle a dropped card. (LMB released.)
		/// </summary>
		/// <param name="Card">The Card dropped.</param>
		/// <param name="Location">Location at which mouse cursor dropped the card.</param>
		public override MoveStatus HandleDroppedCard(PlayingCard Card, PlayingStack OntoStack) {
			return MoveStatus.Invalid;
		}

		PlayingCard selectedCard = null;
		
		/// <summary>
		/// Denotes that a card has been clicked.
		/// </summary>
		/// <param name="Card">The Card that was clicked on.</param>
		public override MoveStatus HandleClickedCard(PlayingCard Card) {
			if (selectedCard == null) {
            if (Card.Stack.Type == StackTypes.Tableu) {
               if (Card.Number == CardNumbers.King) {
                  _currentMove = new ArrayList();
                  
                  OnCardMove(Card, discard.Location);
                  discard.AddTopCard(Card);
                  fillEmptyTableu();

                  OnValidMove();

                  return MoveStatus.ValidMoveEnd;
               } else {
                  selectedCard = Card;
                  selectedCard.Highlighted = true;
                  return MoveStatus.ValidClick;
               }
            }
            return MoveStatus.Invalid;
         } else {
				MoveStatus mv = MoveStatus.Invalid;
				if ((int)Card.Number + (int)selectedCard.Number == 13) {
               _currentMove = new ArrayList();

               OnCardMove(selectedCard, discard.Location);
               discard.AddTopCard(selectedCard);

               OnCardMove(Card, discard.Location);
					discard.AddTopCard(Card);
					mv = MoveStatus.ValidMoveEnd;
            
               fillEmptyTableu();


               OnValidMove();
            }

				selectedCard.Highlighted = false;
				selectedCard = null;
            return mv;
			}
		}

      /*
      public override void UndoLastMove() {
         if (_undoStack.Count > 0) {
            ArrayList al = (ArrayList)_undoStack[_undoStack.Count - 1];

            for (int i = al.Count - 1; i >=0; i--) {
               UndoMoveCard umc = (UndoMoveCard)al[i];
               umc.StartCard.Stack.RemoveTopCard(umc.StartCard);

               OnCardMove(umc.StartCard, umc.StartStack.Location);

               umc.StartStack.AddTopCard(umc.StartCard);
               umc.StartCard.IsFaceShowing = umc.IsFaceVisibleAtStart;
            }
            _undoStack.Remove(al);
         } else {
            MessageBox.Show("No move to undo");
         }
      }
      */

		private ArrayList _currentMove;
      public ArrayList CurrentMove {
         get { return _currentMove; }
      }

      private ArrayList _undoStack  = new ArrayList();

      public override ArrayList GetValidMoves() {
         ArrayList al = new ArrayList();

         for (int i = 0; i < 10; i++) {
            PlayingCard firstCard = tableu[i].LookTopCard();
            if (firstCard == null)
               continue;

            for (int j = i+1; j < 10; j++) {
               PlayingCard secondCard = tableu[j].LookTopCard();
               if (secondCard == null)
                  continue;

               ValidMoveInfo vmi = null;

               if ((int)firstCard.Number + (int)secondCard.Number == 13) {
                  vmi = new ValidMoveInfo(ValidMoveType.DoubleCardClick);
                  vmi.FirstCard = firstCard;
                  vmi.SecondCard = secondCard;
               }

               if (vmi != null) {
                  addMoveToValidList(ref al, vmi);
               }
            }

            if (tableu[i].LookTopCard().Number == CardNumbers.King) {
               ValidMoveInfo vmi = new ValidMoveInfo(ValidMoveType.SingleCardClick);
               vmi.FirstCard = tableu[i].LookTopCard();
               addMoveToValidList(ref al, vmi);
            }


         }

         return al;
      }

      private bool addMoveToValidList(ref ArrayList validMoves, ValidMoveInfo vmi) {
         bool alreadyAdded = false;
         foreach (ValidMoveInfo existingMove in validMoves) {
         
            if (vmi.FirstCard.Number == CardNumbers.King && existingMove.FirstCard.Number == CardNumbers.King) {
               alreadyAdded = true;
            }

            if (vmi.FirstCard.Number == CardNumbers.King || existingMove.FirstCard.Number == CardNumbers.King) {
               continue;
            }

            if ( (vmi.FirstCard.Number == existingMove.FirstCard.Number &&
               vmi.SecondCard.Number == existingMove.SecondCard.Number ) ||
               (vmi.FirstCard.Number == existingMove.SecondCard.Number &&
               vmi.SecondCard.Number == existingMove.FirstCard.Number) ) {
               alreadyAdded = true;
            }
         }
         if (!alreadyAdded) {
            validMoves.Add(vmi);
         }
         return true;
      }
      
      /// <summary>
		/// Denotes that a card has been clicked.
		/// </summary>
		/// <param name="Card">The Card that was clicked on.</param>
		public override MoveStatus HandleDoubleClickedCard(PlayingCard Card) {
			return MoveStatus.Invalid;
		}

		/// <summary>
		/// Denotes that a empty stack has been clicked.
		/// </summary>
		/// <param name="Stack">The stack that received the click event.</param>
		public override MoveStatus HandleClickedStack(PlayingStack Stack) {
			return MoveStatus.Invalid;
		}

		/// <summary>
		/// Denotes that a empty stack has been double clicked.
		/// </summary>
		/// <param name="Stack">The stack that received the click event.</param>
		public override MoveStatus HandleDoubleClickedStack(PlayingStack Stack) {
			return MoveStatus.Invalid;
		}

		/// <summary>
		/// Used on mouse down to determine if the selected card can move.
		/// </summary>
		/// <param name="Card">The Card to check</param>
		/// <returns>True if it can be moved (dragged), false otherwise.</returns>
		public override bool CanCardMove(PlayingCard Card) {
			return false;
		}

		/// <summary>
		/// Used to check for Victory Conditions.
		/// </summary>
		/// <returns>Returns true if Victory Conditions have all been met.</returns>
		public override bool CheckForVictory() {
			if (discard.CardCount == 52) {
				return true;
			} else {
				return false;
			}
		}

      public override int Score { 
			get { return -50 + (discard.CardCount * 5); }
		}
   }
}
