using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using Solitare.Rules;

namespace Solitare {
   /// <summary>
   /// Summary description for Form1.
   /// </summary>
   public class CardGame : System.Windows.Forms.Form {
      /// <summary>
      /// Required designer variable.
      /// </summary>
      private System.ComponentModel.Container components = null;

      IRules gameRules;

      internal static Size cardSize = new System.Drawing.Size(90, 150);
      internal static Size stackSize = new System.Drawing.Size(92, 152);

      public CardGame() {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         //
         // TODO: Add any constructor code after InitializeComponent call
         //

         gameRules = (IRules)(new Klondike() );
         
         CardPanel[] cards;
         CardStack deck;

         gameRules.BuildDeck(out cards, out deck);
         
         ArrayList oldDeck = new ArrayList();
         
         for (int i = 0; i < cards.Length; i++) {
            CardPanel p = cards[i];
            p.BackColor = System.Drawing.SystemColors.ActiveCaption;
            p.Location = new System.Drawing.Point(10, 400);
            p.TabIndex = 0;
            p.MouseMove += new System.Windows.Forms.MouseEventHandler(this.OnMouseMove);
            p.MouseDown += new System.Windows.Forms.MouseEventHandler(this.OnMouseDown);
            p.MouseUp += new System.Windows.Forms.MouseEventHandler(this.OnMouseUp);

            oldDeck.Add(p);

            Controls.Add(p);
         }
         

         Random r = new Random();

         deck.DrawTiled = false;
         deck.Location = new System.Drawing.Point(10, 10);
         this.Controls.Add(deck);

         for (int i = 0; i < 52; i++) {
            int j = r.Next(oldDeck.Count);

            CardPanel nextCard = (CardPanel)oldDeck[j];
            if (nextCard == null) {
               Console.WriteLine("Card is null:" + i + ":" + j);
            }

            deck.AddTopCard((CardPanel)oldDeck[j]);
            oldDeck.RemoveAt(j);
         }

         gameRules.LayoutBoard(this);
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if (components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

		#region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         // 
         // CardGame
         // 
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.BackColor = System.Drawing.Color.Green;
         this.ClientSize = new System.Drawing.Size(712, 582);
         this.Name = "CardGame";
         this.Text = "Card Game";
         this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.OnMouseDown);
         this.DoubleClick += new System.EventHandler(this.OnDoubleClick);
         this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.OnMouseUp);
         this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.OnMouseMove);

      }
		#endregion

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main() {
         Application.Run(new CardGame());
      }

      // TODO Abstract out more logic for what to do on Down.
      // TODO Figure out how to draw a moving stack.
      private void OnMouseDown(object sender, System.Windows.Forms.MouseEventArgs e) {
         _currentControl = (Control)sender;
         _currentCanMove = false;
         if (sender is CardPanel) {
            CardPanel p = (CardPanel)sender;

            _x = e.X;
            _y = e.Y;

            _currentX = ((CardPanel)sender).Location.X;
            _currentY = ((CardPanel)sender).Location.Y;

            _currentCanMove = gameRules.CanCardMove(p);
            //            p.BringToFront();
         }
      }

      // TODO Abstract out how to handle a card movement
      private void OnMouseMove(object sender, System.Windows.Forms.MouseEventArgs e) {
         if (!_currentCanMove)
            return;

         if (sender is CardPanel && e.Button == MouseButtons.Left) {
            CardPanel p = (CardPanel)sender;
            
            p.SetBounds(p.Location.X + e.X - _x, p.Location.Y + e.Y - _y,
               p.Width, p.Height);
         }
      }

      // TODO Abstract out more logic from here...
      private void OnMouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
         this.SuspendLayout();
         if (sender is CardPanel && e.Button == MouseButtons.Left) {
            CardPanel p = (CardPanel)sender;
            int x = p.Location.X + e.X - _x;
            int y = p.Location.Y + e.Y - _y;

            if (p.Location.X == _currentX && p.Location.Y == _currentY) {
               gameRules.HandleClickedCard(p);
               return;
            }

            bool goodDrop = gameRules.HandleDroppedCard(p, new Point(x, y));
            
            if (goodDrop) {
               x = p.Location.X;
               y = p.Location.Y;
            } else {
               x = _currentX;
               y = _currentY;
            }
            p.SetBounds(x, y, p.Width, p.Height);
         } else {
            _x = e.X;
            _y = e.Y;
         }
         this.ResumeLayout();
      }

      int _x;
      int _y;

      int _currentX;
      int _currentY;

      Control _currentControl;
      bool _currentCanMove;

      private void OnDoubleClick(object sender, System.EventArgs e) {
         CardGame f = (CardGame)sender;
         Console.WriteLine("LOCATION:" + f._x + ":" + f._y);
      }
   }
}