using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;

namespace Solitare {

   public class CardStack : Panel {
      private bool _drawTiled;
      internal bool DrawTiled {
         get { return _drawTiled; }
         set { _drawTiled = value; }
      }

      public new Point Location {
         set { 
            Point p = value;
            p.X -= 1;
            p.Y -= 1;
            base.Location = p;
         }

         get { 
            Point p = base.Location;
            p.X += 1;
            p.Y += 1;
            return p;
         }
      }

      private Stack _cards = new Stack();

      internal CardPanel GetTopCard() {
         if (_cards.Count > 0) {
            return (CardPanel)_cards.Pop();
         } else 
            return null;
      }

      internal CardPanel LookTopCard() {
         if (_cards.Count > 0) {
            return (CardPanel)_cards.Peek();
         } else {
            return null;
         }
      }

      internal bool RemoveCard(CardPanel Card) {
         if (_cards.Peek() == Card) {
            _cards.Pop();
            return true;
         } else {
            return false;
         }
      }

      internal void AddTopCard(CardPanel Card) {
         if (Card.Stack != null) {
            Card.Stack.RemoveCard(Card);
         }
         _cards.Push(Card);
         Card.BringToFront();
         Card.Location = this.Location;
         if (_drawTiled) {
            int offset = (_cards.Count -1) * 30;
            Card.Location = new Point(Card.Location.X, Card.Location.Y + offset);
         }
         Card.Stack = this;
      }

      internal CardStack() : base() {
         BackColor = Color.Transparent;
         Size = CardGame.stackSize;
      }

      protected override void OnPaint(PaintEventArgs e) {
         Graphics g = e.Graphics;
         this.ForeColor = Color.Black;
         Pen p = new Pen(Color.Red);
         g.DrawLine(p, new Point(0,0), new Point(0, this.Height - 1));
         g.DrawLine(p, new Point(0,0), new Point(this.Width - 1, 0));

         g.DrawLine(p, new Point(0, this.Height - 1), new Point(this.Width - 1, this.Height - 1));
         g.DrawLine(p, new Point(this.Width - 1, 0), new Point(this.Width - 1, this.Height - 1));
      }
   }
}