using System;
using System.Windows.Forms;
using System.Drawing;

namespace Solitare {

   internal enum CardSuites {
      Hearts = 0, 
      Diamonds = 1, 
      Clubs = 2, 
      Spades = 3
   }

   internal enum CardNumbers : int {
      Ace = 1, 
      Two = 2, 
      Three = 3, 
      Four = 4 , 
      Five = 5, 
      Six = 6, 
      Seven = 7, 
      Eight = 8, 
      Nine = 9, 
      Ten = 10, 
      Jack = 11, 
      Queen = 12, 
      King = 13
   }

   public class CardPanel : Panel {
      private CardSuites _suite;
      internal CardSuites Suite {
         get { return _suite; }
      }

      private CardNumbers _number;
      internal CardNumbers Number {
         get { return _number; }
         set { _number = value; }
      }

      // All cards start face down
      private bool _isFaceShowing = false;
      internal bool IsFaceShowing {
         get { return _isFaceShowing; }
         set { _isFaceShowing = value; }
      }

      // No card can move by default
      private bool _canMove = false;
      internal bool CanMove {
         get { return _canMove; }
         set { _canMove = value; }
      }

      // No card can flip by default
      private bool _canFlip = false;
      internal bool CanFlip {
         get { return _canFlip; }
         set { _canFlip = value; }
      }

      public override String ToString() {
         return base.ToString() + ": " + _number.ToString() + ":" + _suite.ToString();
      }

      internal CardPanel(CardSuites Suite, CardNumbers Number) {
         _suite = Suite;
         _number = Number;

         Size = CardGame.cardSize;
      }

      public bool IsTopCard {
         get { return _stack.LookTopCard() == this; }
      }
      
      private CardStack _stack;
      internal CardStack Stack {
         get { return _stack; }
         set { _stack = value; }
      }

      protected override void OnPaint(PaintEventArgs e) {
         Graphics g = e.Graphics;
         Pen p = new Pen(Color.Black, 1);

         if (_isFaceShowing) {
            Color backColor = Color.Wheat;
            Color foreColor;
            if (_suite == CardSuites.Hearts || _suite == CardSuites.Diamonds) {
               foreColor  = Color.Red;
            } else {
               foreColor = Color.Black;
            }
            Font font = new Font("Times New Roman", 26);
            g.FillRectangle(new SolidBrush(backColor), ClientRectangle);
            p = new Pen(Color.Black);
            String number = "";
            switch (_number) {
               case CardNumbers.Two:
               case CardNumbers.Three:
               case CardNumbers.Four:
               case CardNumbers.Five:
               case CardNumbers.Six:
               case CardNumbers.Seven:
               case CardNumbers.Eight:
               case CardNumbers.Nine:
               case CardNumbers.Ten:
                  number = ((int)_number).ToString();
                  break;
               case CardNumbers.Jack:
               case CardNumbers.Queen:
               case CardNumbers.King:
               case CardNumbers.Ace:
                  number = (_number.ToString()[0]).ToString();
                  break;
            }

            String suite = _suite.ToString()[0].ToString();
            g.DrawString(number + " " + suite, font, new SolidBrush(foreColor), 5, 5);
         } else {
            Color backColor = Color.Blue;

            p.Color = Color.White;

            g.FillRectangle(new SolidBrush(backColor), ClientRectangle);
            
            for (int i = 10; i < this.Width; i += 10) {
               g.DrawLine(p, new Point(i, 0), new Point(i, this.Height));
            }
            for (int j = 10; j < this.Height; j += 10) {
               g.DrawLine(p, new Point(0, j), new Point(this.Width, j));
            }

            p.Color = Color.Black;
         }
         g.DrawLine(p, new Point(0,0), new Point(0, this.Height - 1));
         g.DrawLine(p, new Point(0,0), new Point(this.Width - 1, 0));

         g.DrawLine(p, new Point(0, this.Height - 1), new Point(this.Width - 1, this.Height - 1));
         g.DrawLine(p, new Point(this.Width - 1, 0), new Point(this.Width - 1, this.Height - 1));
      }
   }
}