using System.Windows.Forms;
using System.Drawing;

namespace Solitare {
   // TODO I'd like this to handle Panel, not Form
   
   internal interface IRules {
      /// <summary>
      /// Used to build the initial Deck. Returns a reference to the internal Card Stack which
      /// will be shuffled and added to the Board.
      /// </summary>
      /// <param name="Cards">An out array of Cards which will be added the the board, and shuffled into the deck.</param>
      /// <param name="Deck">A reference to the internal Deck object.</param>
      void BuildDeck(out CardPanel[] Cards, out CardStack Deck);

      /// <summary>
      /// Draws the board, laying out where the deck goes, where the stacks go, and initial cards.
      /// </summary>
      /// <param name="Board">The Board Control that the stacks will be added to.</param>
      void LayoutBoard(Form Board);
      
      /// <summary>
      /// Determines how best to handle a dropped card. (LMB released.)
      /// </summary>
      /// <param name="Card">The Card dropped.</param>
      /// <param name="Location">Location at which mouse cursor dropped the card.</param>
      /// <returns>If true, the card was handled. If false, return the card to it's original location.</returns>
      bool HandleDroppedCard(CardPanel Card, Point Location);

      /// <summary>
      /// Denotes that a card has been clicked.
      /// </summary>
      /// <param name="Card">The Card that was clicked on.</param>
      void HandleClickedCard(CardPanel Card);
      
      /// <summary>
      /// Used on mouse down to determine if the selected card can move.
      /// </summary>
      /// <param name="Card">The Card to check</param>
      /// <returns>True if it can be moved (dragged), false otherwise.</returns>
      bool CanCardMove(CardPanel Card);
   }
}