using System;
using System.Windows.Forms;

namespace Solitare.Rules {
   /// <summary>
   /// Summary description for Klondike.
   /// </summary>
   public class Klondike : IRules {
      CardStack deck;
      CardStack discard;

      CardStack[] mainStacks;
      CardStack[] baseStacks;
      
      
      
      public Klondike() {
         //
         // TODO: Add constructor logic here
         //
      }
   
      public void BuildDeck(out CardPanel[] Cards, out CardStack Deck) {
         deck = new CardStack();

         Deck = deck;

         Cards = new CardPanel[52];

         int count = 0;
         for (int i = 0; i < 4; i++) {
            for (int j = 1; j <= 13; j++) {
               CardPanel p = new CardPanel((CardSuites)i, (CardNumbers)j);
               p.Name = p.Number.ToString() + "_" + p.Suite;
               Cards[count++] = p;
            }
         }
      }

      public void LayoutBoard(Form Board) {
         discard= new CardStack();
         discard.DrawTiled = false;
         discard.Location = new System.Drawing.Point(110, 10);
         Board.Controls.Add(discard);

         baseStacks = new CardStack[4];
         for (int i = 0; i < 4; i++) {
            baseStacks[i] = new CardStack();
            baseStacks[i].DrawTiled = false;
            baseStacks[i].Location = new System.Drawing.Point(310 + 100 * i, 10);
            Board.Controls.Add(baseStacks[i]);
         }

         mainStacks = new CardStack[7];
         for (int i = 0; i < 7; i++) {
            mainStacks[i] = new CardStack();
            mainStacks[i].DrawTiled = true;
            mainStacks[i].Location = new System.Drawing.Point(10 + 100 *i, 200);
            Board.Controls.Add(mainStacks[i]);
         }

         for (int i = 0; i < 7; i++) {
            bool first = true;
            for (int j = i; j < 7; j++) {
               CardPanel c = deck.GetTopCard();
               if (c == null) {
                  Console.WriteLine("HELP!{0}:{1}", i, j);
               } else {
                  if (first) {
                     c.IsFaceShowing = true;
                     first = false;
                  }
                  mainStacks[j].AddTopCard(c);
               }
            }
         }
      }

      public bool CanCardMove(CardPanel Card) {
         if (Card.Stack == deck)
            return false;
         
         if (Card == Card.Stack.LookTopCard() ) {
            return true;
         }

         if (Card.IsFaceShowing)
            return true;
         
         return false;

      }
      
      public void HandleClickedCard(CardPanel Card) {
         if (Card.Stack == deck) {
            // If it's the deck, we handle it one way
            for (int i = 0; i < 3; i++) {
               CardPanel newCard = deck.GetTopCard();
               if (newCard != null) {
                  newCard.IsFaceShowing = true;
                  discard.AddTopCard(newCard);
               }
            }
         
         } else if (Card.CanFlip || Card.IsTopCard) {
            Card.IsFaceShowing = true;
            Card.Invalidate();
         }
      }
      
      public bool HandleDroppedCard(CardPanel Card, System.Drawing.Point Location) {
         int x = Location.X;
         int y = Location.Y;

         
         CardStack s = null;
         for (int i = 0; i < 4 && s == null; i++) {
            CardPanel top = baseStacks[i].LookTopCard();
            if ( (top != null && top.Bounds.Contains(x,y) ) ||
               (top == null && baseStacks[i].Bounds.Contains(x, y)) ) {
               s = baseStacks[i];
            }
         }

         for (int i = 0; i < 7 && s == null; i++) {
            CardPanel top = mainStacks[i].LookTopCard();
            if ( (top != null && top != Card && top.Bounds.Contains(x,y) )  ||
               (top == null && mainStacks[i].Bounds.Contains(x, y) ) ) {
               s = mainStacks[i];
            }
         }

         if (s != null) {
            s.AddTopCard(Card);
            return true;
         } else {
            return false;
         }
      }
   }
}