using System;
using System.Collections;

namespace Solitare
{
   public enum ValidMoveType {
      SingleCardClick,
      DoubleCardClick,
      SingleCardToStack,
      StackClick
   }
   
   public class ValidMoveInfo {
      public ValidMoveInfo(ValidMoveType MoveType) {
         _moveType = MoveType;
      }

      private ValidMoveType _moveType;
      /// <summary>
      /// What type of move is about to be executed.
      /// </summary>
      public ValidMoveType MoveType {
         get { return _moveType; }
      }
      
      private PlayingCard _firstCard;
      /// <summary>
      /// What is the first card "clicked" on, or highlighted.
      /// </summary>
      public PlayingCard FirstCard {
         get { return _firstCard; }
         set { _firstCard = value; }
      }

      private PlayingCard _secondCard;
      /// <summary>
      /// If to cards are clicked for this move, this denotes the second card. (DoubleCardClick)
      /// </summary>
      public PlayingCard SecondCard {
         get { return _secondCard; }
         set { _secondCard = value; }
      }

      private PlayingStack _firstStack;
      public PlayingStack FirstStack {
         get { return _firstStack; }
         set { _firstStack = value; }
      }
      
      private bool _allBranchesTested = false;
      /// <summary>
      /// If all of the submoves have been testing, this is true.
      /// </summary>
      public bool AllBranchesTested {
         get { return _allBranchesTested; }
         set { _allBranchesTested = value; }
      }

      private bool _winningBranch = false;
      /// <summary>
      /// Only pertinent if AllBranchesTested == true or it is true, this denotes that
      /// this is part of a winning move set.
      /// </summary>
      public bool WinningBranch {
         get { return _winningBranch; }
         set { _winningBranch = value; }
      }

      private ArrayList _subMoveBranches = null;
      /// <summary>
      /// After this move has been exectued, what moves are available. (Null if this move hasn't been executed)
      /// </summary>
      public ArrayList SubMoveBranches {
         get { return _subMoveBranches; }
         set { _subMoveBranches = value; }
      }

      private String _layoutHash;
      /// <summary>
      /// Used to denote the state of all the cards. Used to avoid (hopefully) circular references.
      /// </summary>
      public String LayoutHash {
         get { return _layoutHash; }
         set { _layoutHash = value; }
      }

   }
}
